// 
// 
// 

#include "colorRecognition.h"
#include <Wire.h>

void ColorRecognitionClass::begin()
{
	Wire.begin();
}

void ColorRecognitionClass::startConvert(void)
{
	/* ָ */
	Wire.beginTransmission(COLOR_SLAVE_ADDRESS);
	Wire.write(1);
	Wire.endTransmission();
}

boolean ColorRecognitionClass::getStatus(void)
{
	uint8_t status;
	/* ѯģ״̬ */
	Wire.beginTransmission(COLOR_SLAVE_ADDRESS);
	Wire.write(2);
	Wire.endTransmission();

	Wire.requestFrom(COLOR_SLAVE_ADDRESS, 1);
	while (Wire.available())
		status = Wire.read();
	if (status == 3)
		return true;
	else
		return false;
}

void ColorRecognitionClass::getRGB(uint8_t &R, uint8_t &G, uint8_t &B)
{
	int i = 0;
	uint8_t temp[6];
	uint8_t state;
	uint16_t r, g, b;

	Wire.beginTransmission(COLOR_SLAVE_ADDRESS);
	Wire.write(3);
	Wire.endTransmission();

	/* ȡ */
	Wire.requestFrom(COLOR_SLAVE_ADDRESS, 6);
	while (Wire.available()) {
		temp[i] = Wire.read();
		i++;
	}
	g = temp[1];
	g = (g << 8) + temp[0];
	b = temp[3];
	b = (b << 8) + temp[2];
	r = temp[5];
	r = (r << 8) + temp[4];

	R = uint16_t((float(r) / 1023) * 255);
	G = uint16_t((float(g) / 1023) * 255);
	B = uint16_t((float(b) / 1023) * 255);

	///* ֹͣ */
	//  Wire.beginTransmission(10); 
	//  Wire.write(3);              
	//  Wire.endTransmission();    
}

void ColorRecognitionClass::colourBalance(uint8_t &R, uint8_t &G, uint8_t &B, float &scale_r, float &scale_g, float &scale_b)
{
	uint8_t maxVal;
	maxVal = max(R, G);
	maxVal = max(maxVal, B);

	if (maxVal == R)
	{
		scale_r = 1;
		scale_g = ((float)R) / G;
		scale_b = ((float)R) / B;
	}
	else if (maxVal == G)
	{
		scale_r = ((float)G) / R;
		scale_g = 1;
		scale_b = ((float)G) / B;
	}
	else if (maxVal == B)
	{
		scale_r = ((float)B) / R;
		scale_g = ((float)B) / G;
		scale_b = 1;
	}

}

void ColorRecognitionClass::RGB2HSV(uint8_t R, uint8_t G, uint8_t B, uint16_t &H, float &S, float &V)
{

	float del_R, del_G, del_B, temp_h;
	uint8_t var_R, var_G, var_B, var_Min, var_Max, del_Max;

	var_R = R;
	var_G = G;
	var_B = B;

	var_Min = min(var_R, var_G);
	var_Min = min(var_Min, var_B);

	var_Max = max(var_R, var_G);
	var_Max = max(var_Max, var_B);

	del_Max = var_Max - var_Min;             //Delta RGB value

	V = var_Max/255.0;

	if (del_Max == 0)                     //This is a gray, no chroma...
	{
		H = 0;
		S = 0;
	}
	else                                    //Chromatic data...
	{
		S = (float)del_Max / var_Max;

		del_R = (((var_Max - var_R) / 6.0) + (del_Max / 2.0)) / del_Max;
		del_G = (((var_Max - var_G) / 6.0) + (del_Max / 2.0)) / del_Max;
		del_B = (((var_Max - var_B) / 6.0) + (del_Max / 2.0)) / del_Max;

		if (var_R == var_Max)
			temp_h = del_B - del_G;

		else if (var_G == var_Max)
			temp_h = (1.0 / 3.0) + del_R - del_B;
		else if (var_B == var_Max) 
			temp_h = (2.0 / 3.0) + del_G - del_R;

		if (temp_h < 0) temp_h += 1;
		if (temp_h > 1) temp_h -= 1;
	}
	H = (uint16_t)(temp_h*360);
}

void ColorRecognitionClass::HSV2RGB(uint16_t H, float S, float V, uint8_t  &R, uint8_t &G, uint8_t &B)
{
	uint8_t hi;
	float f, p, q, t;
	hi = (uint8_t)(H / 60);
	f = (float)H / 60 - hi;
	p = V * (1 - S);
	q = V * (1 - f * S);
	t = V * (1 - (1 - f) * S);
	switch (hi)
	{
		//0: {r,g,b} = {v,t,p}
	case 0:
		R = V * 255;
		G = t * 255;
		B = p * 255;
		break;
		//1: {r,g,b} = {q,v,p}
	case 1:
		R = q * 255;
		G = V * 255;
		B = p * 255;
		break;
		//2: {r,g,b} = {p,v,t}
	case 2:
		R = p * 255;
		G = V * 255;
		B = t * 255;
		break;
		//3: {r,g,b} = {p,q,v}
	case 3:
		R = p * 255;
		G = q * 255;
		B = V * 255;
		break;
		//4: {r,g,b} = {t,p,v}
	case 4:
		R = t * 255;
		G = p * 255;
		B = V * 255;
		break;
		//5: {r,g,b} = {v,p,q}
	case 5:
		R = V * 255;
		G = p * 255;
		B = q * 255;
		break;
	default:
		break;
	}
}

//ColorRecognitionClass ColorRecognition;

