/* duinoplusBoxII 生产测试例程
测试项目：
1、外接电源输入电压测试
2、USB供电及通信测试
3、各模块功能测试
4、扩展接口测试
测试方法：
1、外接电源输入电压测试：万用表测测试点电压
2、USB供电及通信测试：跟模块测试一起完成
3、模块及扩展功能测试：
三种状态下完成测试，各状态进入方法:
状态1：固件烧录后缺省进入此状态。
状态2：方向键往上拔，然后按复位键，等显示屏上出现超声波数据后，松开按键。
状态3：方向键往下拔，然后按复位键，待心率血氧传感器上红灯亮时，松开按键。
各状态下测试项目：
状态1：
a、方向键：上下左左拔动按键，OLED对应位置值会显示0-1023数值变化。
b、声音：OLED对应位置值在0-1023范围内变化。
c、遥控接接头：按遥控按键时，OLED屏上相应位置会显示遥控编码值，同时板上接收头旁指示灯闪烁。
d、时钟模块：OLED屏上会显示时间信息。
e、颜色传感器：OLED屏上会显示R G B值。
f、RGB LED：复位后，会依次显示红绿蓝白，注意观察是否有坏点
g、mp3模块：复位后，会播放音乐。同时模块旁指示灯亮。
h、步进电机：复位后，舵机旋转。
i、OLED屏：能正确显示各数据。
状态2：
测试超声波模块：观察测试数据是否正常
状态3：
a、心率血氧模块测试：打开串口软件，波特率设为9600，手指轻放在模块上，串口软件上能正确显示心率数据。
b、扩展接口测试：数码管接在A5 A4对应的接口，按键模块接在5 6对应接口，分别按两个按键，数值在1111 2222间切换。

*/
//#include "IRremoteInt.h"
#include "IRremoteTools_1.h"
#include "IRremote_1.h"
#include <Wire.h>
#include <SoftwareSerial.h>
#include "kt540b.h"
#include "Adafruit_NeoPixel.h"
#include "DS3231.h"
#include "colorRecognition.h"
#include "U8glib.h"
#include "MAX30100_PulseOximeter.h"
#include "TM1650.h"

#define DEBUG_DS3231

#define PIN_RGB_LED 8
#define PIN_SOUND   A0
#define PIN_TRIG    5
#define PIN_ECHO    4
#define PIN_RLED    6
#define PIN_GLED    7
#define PIN_BLED    2
#define PIN_COLOR   A3
#define PIN_PS1     A1
#define PIN_PS2     A2

#define REPORTING_PERIOD_MS     1000

U8GLIB_SSD1306_128X64 u8g(U8G_I2C_OPT_NONE | U8G_I2C_OPT_DEV_0);	// I2C / TWI 
Kt540bClass mp3(7, 6);
Adafruit_NeoPixel rgbLed(64, PIN_RGB_LED);
float distance;
unsigned long irData;
uint8_t r, g, b;
uint16_t h;
float s, v;
float scaleR, scaleG, scaleB;
ColorRecognitionClass ColorRecognition;

DS3231 ds3231;
byte year, month, date, dow, hour, minute, second;
bool h12, PM;
uint8_t testFlag = 1;
int volume;
float hr;
uint8_t spo2;
float temperature;

PulseOximeter pox;
uint32_t tsLastReport = 0;

TM1650 digitalDis;

static void draw(void);
static void getColor(void);
static void motorTest(void);
static void getDistanceVal(void);
static void rgbLedTest(void);
static void getIrVal(void);
void onBeatDetected(void);

void setup()
{
	int ps1Val, ps2Val;

	Serial.begin(9600);

	ps1Val = analogRead(PIN_PS1);
	ps2Val = analogRead(PIN_PS2);

	if ((ps1Val > 750) && (ps2Val < 750) && (ps2Val > 250))
		testFlag = 1;     //测时间、声音、摇杆、颜色、步进电机、RGB LED、MP3、红外遥控、两个扩展接口
	else if ((ps2Val > 750) && (ps1Val < 750) && (ps1Val > 250))
		testFlag = 2;    //测超声波
	else if (((ps2Val < 250) && (ps1Val < 750) && (ps1Val > 250)))
		testFlag = 3;   //测心率血氧

	if (testFlag == 1)
	{
		beginIRremote();
		mp3.begin();         // 1. mp3模块测试
		delay(2500);
		mp3.setVolume(20);
		mp3.play(1);

		ds3231.setClockMode(true);
		ds3231.setYear(17);
		ds3231.setMonth(8);
		ds3231.setDate(1);
		ds3231.setDoW(7);
		ds3231.setHour(10);
		ds3231.setMinute(10);
		ds3231.setSecond(0);
		Serial.println(ds3231.getYear());

		ColorRecognition.begin();

		motorTest();         // 2、马达测试
		rgbLedTest();        // 3、RGB LED测试
		u8g.setFont(u8g_font_6x10r);
	}

	else if (testFlag == 2)
	{
		pinMode(PIN_TRIG, OUTPUT);
		pinMode(PIN_ECHO, INPUT);
		u8g.setFont(u8g_font_9x15);
	}
	else if (testFlag == 3)
	{
		pinMode(5, INPUT);
		pinMode(6, INPUT);
		digitalDis.init();
		digitalDis.setBrightness(TM1650_MIN_BRIGHT);

		if (!pox.begin()) {
			Serial.println("FAILED");
			for (;;);
		}
		else {
			Serial.println("SUCCESS");
		}
		pox.setIRLedCurrent(MAX30100_LED_CURR_7_6MA);
		pox.setOnBeatDetectedCallback(onBeatDetected);
		u8g.setFont(u8g_font_9x15);
	}
}

void loop()
{
	if (testFlag == 1)
	{
		getIrVal();
		ds3231.getTime(year, month, date, dow, hour, minute, second);
		getColor();
		volume = analogRead(PIN_SOUND);

		u8g.firstPage();
		do {
			draw();
		} while (u8g.nextPage());
	}
	else if (testFlag == 2)
	{
		getDistanceVal();
		u8g.firstPage();
		do {
			draw();
		} while (u8g.nextPage());
	}
	else if (testFlag == 3)
	{
		if (digitalRead(5))
		{
			digitalDis.displayString("1111");
		}
		if (digitalRead(6))
		{
			digitalDis.displayString("2222");
		}

		pox.update();
		if (millis() - tsLastReport > REPORTING_PERIOD_MS) {

			hr = pox.getHeartRate();
			spo2 = pox.getSpO2();
			temperature = pox.getTemperature();
			tsLastReport = millis();
			Serial.println(hr);
		}
	}
	//u8g.firstPage();
	//do {
	//	draw();
	//} while (u8g.nextPage());
}

static void draw(void)
{
	switch (testFlag)
	{
	case 1:
		u8g.drawStr(0, 10, "VOL: ");     //4、声音测试
		u8g.setPrintPos(30, 10);
		u8g.print(volume);

		u8g.drawStr(0, 25, "PS: ");         //5、摇杆测试
		u8g.setPrintPos(23, 25);
		u8g.print(analogRead(PIN_PS1));
		u8g.setPrintPos(55, 25);
		u8g.print(analogRead(PIN_PS2));

		u8g.drawStr(0, 37, "IR: ");        //6、红外遥控测试
		u8g.setPrintPos(40, 37);
		u8g.print(irData);

		u8g.drawStr(0, 49, "Time:");      //7、时间测试
		u8g.setPrintPos(35, 49);
		u8g.print(year);
		u8g.drawStr(47, 49, "-");
		u8g.setPrintPos(54, 49);
		u8g.print(month);
		u8g.drawStr(65, 49, "-");
		u8g.setPrintPos(71, 49);
		u8g.print(date);
		u8g.setPrintPos(82, 49);
		u8g.print(hour);
		u8g.drawStr(94, 49, ":");
		u8g.setPrintPos(100, 49);
		u8g.print(minute);
		u8g.drawStr(112, 49, ":");
		u8g.setPrintPos(118, 49);
		u8g.print(second);

		u8g.drawStr(0, 62, "Color:");    //8、颜色测试
		u8g.drawStr(40, 62, "R");
		u8g.setPrintPos(50, 62);
		u8g.print(r);
		u8g.drawStr(72, 62, "G");
		u8g.setPrintPos(82, 62);
		u8g.print(g);
		u8g.drawStr(102, 62, "B");
		u8g.setPrintPos(110, 62);
		u8g.print(b);
		break;
	case 2:
		u8g.drawStr(0, 10, "D:  ");
		u8g.setPrintPos(20, 10);
		u8g.print(distance);
		break;
	case 3:
		//u8g.drawStr(0, 16, "HR: ");
		//u8g.setPrintPos(60, 16);
		//u8g.print(hr);
		//Serial.println(hr);
		break;
	default:
		break;
	}
}

static void getColor(void)
{
	ColorRecognition.startConvert();
	delay(90);
	ColorRecognition.getStatus();
	//获取RGB值
	//	if (ColorRecognition.getStatus())
	//	{
	ColorRecognition.getRGB(r, g, b);
	//	}

	Serial.print("g: ");	Serial.print(g);
	Serial.print("  b: ");	Serial.print(b);
	Serial.print("  r: ");	Serial.print(r);
	Serial.println("");
}

static void motorTest(void)
{
	/* 正转 */
	Wire.beginTransmission(0x0E); // transmit to device #8
	Wire.write(1);        // BYTE0:  1: 转速 2：转角 
	Wire.write(2);        // BYTE1: 方向 1：正转 2：反转 3 停止
	Wire.write(0x32);     // BYTE2、BYTE3: 转速或转角的低、高位字节。转速：0-100 转角：0-360
	Wire.write(0);              // sends one byte 
	Wire.endTransmission();    // stop transmitting  
	delay(2000);
	/* 反转 */
	Wire.beginTransmission(0x0E); // transmit to device #8
	Wire.write(1);        // BYTE0:  1: 转速 2：转角 
	Wire.write(1);        // BYTE1: 方向 1：正转 2：反转 3 停止
	Wire.write(0x32);     // BYTE2、BYTE3: 转速或转角的低、高位字节。转速：0-100 转角：0-360
	Wire.write(0);              // sends one byte 
	Wire.endTransmission();    // stop transmitting 
	delay(2000);
	/* 停止 */
	Wire.beginTransmission(0x0E); // transmit to device #8
	Wire.write(1);        // BYTE0:  1: 转速 2：转角 3 停止
	Wire.write(1);        // BYTE1: 方向 1：正转 2：反转
	Wire.write(0);     // BYTE2、BYTE3: 转速或转角的低、高位字节。转速：0-100 转角：0-360
	Wire.write(0);              // sends one byte 
	Wire.endTransmission();    // stop transmitting 
	delay(2000);
	/* 正转360度*/
	Wire.beginTransmission(0x0E); // transmit to device #8
	Wire.write(2);        // BYTE0:  1: 转速 2：转角
	Wire.write(2);        // BYTE1: 方向 1：正转 2：反转
	Wire.write(0x68);     // BYTE2、BYTE3: 转速或转角的低、高位字节。转速：0-100 转角：0-360
	Wire.write(0x01);              // sends one byte 
	Wire.endTransmission();    // stop transmitting  
	delay(2000);
}

static void getDistanceVal(void)
{
	digitalWrite(PIN_TRIG, LOW);
	delayMicroseconds(2);
	digitalWrite(PIN_TRIG, HIGH);
	delayMicroseconds(10);
	digitalWrite(PIN_TRIG, LOW);
	// 检测脉冲宽度，并计算出距离
	distance = pulseIn(PIN_ECHO, HIGH) / 58.00;
	Serial.print(distance);
	Serial.print("cm");
	Serial.println();
}

static void rgbLedTest(void)
{
	pinMode(PIN_RGB_LED, OUTPUT);
	rgbLed.begin();
	rgbLed.setBrightness(50);

	for (int i = 0; i < 64; i++)
		rgbLed.setPixelColor(i, 50, 0, 0);
	rgbLed.show();
	delay(500);
	for (int i = 0; i < 64; i++)
		rgbLed.setPixelColor(i, 0, 50, 0);
	rgbLed.show();
	delay(500);
	for (int i = 0; i < 64; i++)
		rgbLed.setPixelColor(i, 0, 0, 50);
	rgbLed.show();
	delay(500);
	for (int i = 0; i < 64; i++)
		rgbLed.setPixelColor(i, 50, 50, 50);
	rgbLed.show();
	delay(500);
	rgbLed.clear();
	rgbLed.show();
}

static void getIrVal(void)
{
	if (IRrecived())
	{
		irData = getIRresult();
		Serial.println(irData);
		resumeIRremote();
	}
}

void onBeatDetected()
{
	//Serial.println("Beat!");
}